Index: channels/chan_zap.c
===================================================================
--- channels/chan_zap.c	(revision 90990)
+++ channels/chan_zap.c	(working copy)
@@ -552,6 +552,7 @@
 	unsigned int transfer:1;
 	unsigned int use_callerid:1;			/*!< Whether or not to use caller id on this channel */
 	unsigned int use_callingpres:1;			/*!< Whether to use the callingpres the calling switch sends */
+	unsigned int mwicall:1;				/*!< Call made for the sole purpose of sending mwi */
 	unsigned int usedistinctiveringdetection:1;
 	unsigned int zaptrcallerid:1;			/*!< should we use the callerid from incoming call on zap transfer or not */
 	unsigned int transfertobusy:1;			/*!< allow flash-transfers to busy channels */
@@ -2089,7 +2090,10 @@
 				}
 				p->callwaitcas = 0;
 				if ((p->cidspill = ast_malloc(MAX_CALLERID_SIZE))) {
-					p->cidlen = ast_callerid_generate(p->cidspill, ast->cid.cid_name, ast->cid.cid_num, AST_LAW(p));
+					if (p->mwicall)
+						p->cidlen = vmwi_generate(p->cidspill, has_voicemail(p), 1, AST_LAW(p));
+					else
+						p->cidlen = ast_callerid_generate(p->cidspill, ast->cid.cid_name, ast->cid.cid_num, AST_LAW(p));
 					p->cidpos = 0;
 					send_callerid(p);
 				}
@@ -3196,6 +3200,7 @@
 		restart_monitor();
 	}
 
+	p->mwicall = 0;
 	p->callwaitingrepeat = 0;
 	p->cidcwexpire = 0;
 	p->oprmode = 0;
@@ -4649,6 +4654,13 @@
 			p->subs[index].f.subclass = AST_CONTROL_RINGING;
 			break;
 		case ZT_EVENT_RINGERON:
+			/* 
+			 * For MWI indication we should stop the ringer after cidspill is sent
+			 * because rings are not allowed after mwi cidspill
+			 */
+			if ((ast->rings == p->cidrings) && p->mwicall) {
+				zt_set_hook(p->subs[index].zfd, ZT_RINGOFF);
+			}
 			break;
 		case ZT_EVENT_NOALARM:
 			p->inalarm = 0;
@@ -8610,6 +8622,12 @@
 				if (opt == 'c') {
 					/* Confirm answer */
 					p->confirmanswer = 1;
+				} else if (opt == 'm') {
+					/*
+					 * Instead of sending callerid only
+					 * we will be sending message waiting indicator also
+					 */
+					p->mwicall = 1;
 				} else if (opt == 'r') {
 					/* Distinctive ring */
 					if (res < 3)
